@@ -1,20 +1,3 @@
-# This file is part of the python-chess library.
-# Copyright (C) 2015 Jean-Noël Avila <jn.avila@free.fr>
-# Copyright (C) 2015-2021 Niklas Fiekas <niklas.fiekas@backscattering.de>
-#
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program. If not, see <http://www.gnu.org/licenses/>.
-
 from __future__ import annotations
 
 import ctypes
@@ -161,10 +144,9 @@ FLIPT = init_flipt()
 
 
 def init_pp48_idx() -> Tuple[List[List[int]], List[int], List[int]]:
-    MAX_I = 48
-    MAX_J = 48
+    MAX_I = MAX_J = 48
     idx = 0
-    pp48_idx = [[-1] * MAX_J for i in range(MAX_I)]
+    pp48_idx = [[-1] * MAX_J for _ in range(MAX_I)]
     pp48_sq_x = [NOSQUARE] * MAX_PP48_INDEX
     pp48_sq_y = [NOSQUARE] * MAX_PP48_INDEX
 
@@ -187,10 +169,8 @@ PP48_IDX, PP48_SQ_X, PP48_SQ_Y = init_pp48_idx()
 
 
 def init_ppp48_idx() -> Tuple[List[List[List[int]]], List[int], List[int], List[int]]:
-    MAX_I = 48
-    MAX_J = 48
-    MAX_K = 48
-    ppp48_idx = [[[-1] * MAX_I for j in range(MAX_J)] for k in range(MAX_K)]
+    MAX_I = MAX_J = MAX_K = 48
+    ppp48_idx = [[[-1] * MAX_I for _ in range(MAX_J)] for _ in range(MAX_K)]
     ppp48_sq_x = [NOSQUARE] * MAX_PPP48_INDEX
     ppp48_sq_y = [NOSQUARE] * MAX_PPP48_INDEX
     ppp48_sq_z = [NOSQUARE] * MAX_PPP48_INDEX
@@ -227,7 +207,7 @@ PPP48_IDX, PPP48_SQ_X, PPP48_SQ_Y, PPP48_SQ_Z = init_ppp48_idx()
 
 
 def init_aaidx() -> Tuple[List[int], List[List[int]]]:
-    aaidx = [[-1] * 64 for y in range(64)]
+    aaidx = [[-1] * 64 for _ in range(64)]
     aabase = [0] * MAX_AAINDEX
 
     idx = 0
@@ -257,7 +237,7 @@ def init_aaa() -> Tuple[List[int], List[List[int]]]:
         aaa_base[a + 1] = accum
 
     # Get aaa_xyz.
-    aaa_xyz = [[-1] * 3 for idx in range(MAX_AAAINDEX)]
+    aaa_xyz = [[-1] * 3 for _ in range(MAX_AAAINDEX)]
 
     idx = 0
     for z in range(64):
@@ -336,7 +316,7 @@ def wsq_to_pidx48(pawn: int) -> int:
     return idx48
 
 def init_ppidx() -> Tuple[List[List[int]], List[int], List[int]]:
-    ppidx = [[-1] * 48 for i in range(24)]
+    ppidx = [[-1] * 48 for _ in range(24)]
     pp_hi24 = [-1] * MAX_PPINDEX
     pp_lo48 = [-1] * MAX_PPINDEX
 
@@ -396,7 +376,7 @@ def norm_kkindex(x: chess.Square, y: chess.Square) -> Tuple[int, int]:
     return x, y
 
 def init_kkidx() -> Tuple[List[List[int]], List[int], List[int]]:
-    kkidx = [[-1] * 64 for x in range(64)]
+    kkidx = [[-1] * 64 for _ in range(64)]
     bksq = [-1] * MAX_KKINDEX
     wksq = [-1] * MAX_KKINDEX
     idx = 0
@@ -1376,51 +1356,9 @@ iWMATEt = tb_WMATE | 4
 iBMATEt = tb_BMATE | 4
 
 
-def removepiece(ys: List[int], yp: List[int], j: int) -> None:
-    del ys[j]
-    del yp[j]
-
 def opp(side: int) -> int:
     return 1 if side == 0 else 0
 
-def adjust_up(dist: int) -> int:
-    udist = dist
-    sw = udist & INFOMASK
-
-    if sw in [iWMATE, iWMATEt, iBMATE, iBMATEt]:
-        udist += (1 << PLYSHIFT)
-
-    return udist
-
-def bestx(side: int, a: int, b: int) -> int:
-    # 0 = selectfirst
-    # 1 = selectlowest
-    # 2 = selecthighest
-    # 3 = selectsecond
-    comparison = [
-        # draw, wmate, bmate, forbid
-        [0, 3, 0, 0],  # draw
-        [0, 1, 0, 0],  # wmate
-        [3, 3, 2, 0],  # bmate
-        [3, 3, 3, 0],  # forbid
-    ]
-
-    xorkey = [0, 3]
-
-    if a == iFORBID:
-        return b
-    if b == iFORBID:
-        return a
-
-    retu = [a, a, b, b]
-
-    if b < a:
-        retu[1] = b
-        retu[2] = a
-
-    key = comparison[a & 3][b & 3] ^ xorkey[side]
-    return retu[key]
-
 def unpackdist(d: int) -> Tuple[int, int]:
     return d >> PLYSHIFT, d & INFOMASK
 
@@ -1512,12 +1450,11 @@ class Request:
     black_piece_types: List[int]
     is_reversed: bool
 
-    def __init__(self, white_squares: List[int], white_types: List[chess.PieceType], black_squares: List[int], black_types: List[chess.PieceType], side: int, epsq: int):
+    def __init__(self, white_squares: List[int], white_types: List[chess.PieceType], black_squares: List[int], black_types: List[chess.PieceType], side: int):
         self.white_squares, self.white_types = sortlists(white_squares, white_types)
         self.black_squares, self.black_types = sortlists(black_squares, black_types)
         self.realside = side
         self.side = side
-        self.epsq = epsq
 
 
 @dataclasses.dataclass
@@ -1556,8 +1493,8 @@ class PythonTablebase:
         Probes for depth to mate information.
 
         The absolute value is the number of half-moves until forced mate
-        (or ``0`` in drawn positions). The value is positive if the
-        side to move is winning, otherwise it is negative.
+        (or ``0`` in drawn or checkmated positions). The value is positive if
+        the side to move is winning, otherwise it is negative or 0.
 
         In the example position, white to move will get mated in 10 half-moves:
 
@@ -1589,17 +1526,37 @@ class PythonTablebase:
         if board.occupied == board.kings:
             return 0
 
+        # Resolve en passant.
+        dtm = self._probe_dtm_no_ep(board)
+        for move in board.generate_legal_ep():
+            try:
+                board.push(move)
+
+                if board.is_checkmate():
+                    child_dtm = 1
+                else:
+                    child_dtm = -self._probe_dtm_no_ep(board)
+                    if child_dtm > 0:
+                        child_dtm += 1
+                    elif child_dtm < 0:
+                        child_dtm -= 1
+
+                dtm = min(dtm, child_dtm) if dtm * child_dtm > 0 else max(dtm, child_dtm)
+            finally:
+                board.pop()
+        return dtm
+
+    def _probe_dtm_no_ep(self, board: chess.Board) -> int:
         # Prepare the tablebase request.
         white_squares = list(chess.SquareSet(board.occupied_co[chess.WHITE]))
         white_types = [typing.cast(chess.PieceType, board.piece_type_at(sq)) for sq in white_squares]
         black_squares = list(chess.SquareSet(board.occupied_co[chess.BLACK]))
         black_types = [typing.cast(chess.PieceType, board.piece_type_at(sq)) for sq in black_squares]
         side = 0 if (board.turn == chess.WHITE) else 1
-        epsq = board.ep_square if board.ep_square else NOSQUARE
-        req = Request(white_squares, white_types, black_squares, black_types, side, epsq)
+        req = Request(white_squares, white_types, black_squares, black_types, side)
 
         # Probe.
-        dtm = self.egtb_get_dtm(req)
+        dtm = self._tb_probe(req)
         ply, res = unpackdist(dtm)
 
         if res == iWMATE:
@@ -1695,10 +1652,7 @@ class PythonTablebase:
             req.white_piece_types = req.black_types
             req.black_piece_squares = [flip_ns(s) for s in req.white_squares]
             req.black_piece_types = req.white_types
-
             req.side = opp(req.side)
-            if req.epsq != NOSQUARE:
-                req.epsq = flip_ns(req.epsq)
         else:
             raise MissingTableError(f"no gaviota table available for: {white_letters.upper()}v{black_letters.upper()}")
 
@@ -1728,77 +1682,6 @@ class PythonTablebase:
             _, stream = self.streams.popitem()
             stream.close()
 
-    def egtb_get_dtm(self, req: Request) -> int:
-        dtm = self._tb_probe(req)
-
-        if req.epsq != NOSQUARE:
-            capturer_a = 0
-            capturer_b = 0
-            xed = 0
-
-            # Flip for move generation.
-            if req.side == 0:
-                xs = list(req.white_piece_squares)
-                xp = list(req.white_piece_types)
-                ys = list(req.black_piece_squares)
-                yp = list(req.black_piece_types)
-            else:
-                xs = list(req.black_piece_squares)
-                xp = list(req.black_piece_types)
-                ys = list(req.white_piece_squares)
-                yp = list(req.white_piece_types)
-
-            # Captured pawn trick: from ep square to captured.
-            xed = req.epsq ^ (1 << 3)
-
-            # Find captured index (j).
-            try:
-                j = ys.index(xed)
-            except ValueError:
-                j = -1
-
-            # Try first possible ep capture.
-            if 0 == (0x88 & (map88(xed) + 1)):
-                capturer_a = xed + 1
-
-            # Try second possible ep capture.
-            if 0 == (0x88 & (map88(xed) - 1)):
-                capturer_b = xed - 1
-
-            if (j > -1) and (ys[j] == xed):
-                # Find capturers (i).
-                for i in range(len(xs)):
-                    if xp[i] == chess.PAWN and (xs[i] == capturer_a or xs[i] == capturer_b):
-                        epscore = iFORBID
-
-                        # Copy position.
-                        xs_after = xs[:]
-                        ys_after = ys[:]
-                        xp_after = xp[:]
-                        yp_after = yp[:]
-
-                        # Execute capture.
-                        xs_after[i] = req.epsq
-                        removepiece(ys_after, yp_after, j)
-
-                        # Flip back.
-                        if req.side == 1:
-                            xs_after, ys_after = ys_after, xs_after
-                            xp_after, yp_after = yp_after, xp_after
-
-                        # Make subrequest.
-                        subreq = Request(xs_after, xp_after, ys_after, yp_after, opp(req.side), NOSQUARE)
-                        try:
-                            epscore = self._tb_probe(subreq)
-                            epscore = adjust_up(epscore)
-
-                            # Choose to ep or not.
-                            dtm = bestx(req.side, epscore, dtm)
-                        except IndexError:
-                            break
-
-        return dtm
-
     def egtb_block_getnumber(self, req: Request, idx: int) -> int:
         maxindex = EGKEY[req.egkey].maxindex
 
