@@ -1,22 +1,7 @@
-# This file is part of the python-chess library.
-# Copyright (C) 2012-2021 Niklas Fiekas <niklas.fiekas@backscattering.de>
-#
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program. If not, see <http://www.gnu.org/licenses/>.
-
 from __future__ import annotations
 
 import abc
+import dataclasses
 import enum
 import itertools
 import logging
@@ -27,15 +12,11 @@ import chess
 import chess.engine
 import chess.svg
 
-from typing import Any, Callable, Dict, Generic, Iterable, Iterator, List, Mapping, MutableMapping, Set, TextIO, Tuple, Type, TypeVar, Optional, Union
+from typing import Any, Callable, Dict, Generic, Iterable, Iterator, List, Literal, Mapping, MutableMapping, Set, TextIO, Tuple, Type, TypeVar, Optional, Union
 from chess import Color, Square
 
-try:
-    from typing import Literal
-    _TrueLiteral = Literal[True]
-except ImportError:
-    # Before Python 3.8.
-    _TrueLiteral = bool  # type: ignore
+if typing.TYPE_CHECKING:
+    from typing_extensions import Self
 
 
 LOGGER = logging.getLogger(__name__)
@@ -164,6 +145,39 @@ SKIP = SkipType.SKIP
 ResultT = TypeVar("ResultT", covariant=True)
 
 
+class TimeControlType(enum.Enum):
+    UNKNOWN = 0
+    UNLIMITED = 1
+    STANDARD = 2
+    RAPID = 3
+    BLITZ = 4
+    BULLET = 5
+
+
+@dataclasses.dataclass
+class TimeControlPart:
+    moves: int = 0
+    time: int = 0
+    increment: float = 0
+    delay: float = 0
+
+
+@dataclasses.dataclass
+class TimeControl:
+    """
+    PGN TimeControl Parser
+    Spec: http://www.saremba.de/chessgml/standards/pgn/pgn-complete.htm#c9.6
+
+    Not Yet Implemented:
+    - Hourglass/Sandclock ('*' prefix)
+    - Differentiating between Bronstein and Simple Delay (Not part of the PGN Spec)
+      - More Info: https://en.wikipedia.org/wiki/Chess_clock#Timing_methods
+    """
+
+    parts: list[TimeControlPart] = dataclasses.field(default_factory=list)
+    type: TimeControlType = TimeControlType.UNKNOWN
+
+
 class _AcceptFrame:
     def __init__(self, node: ChildNode, *, is_variation: bool = False, sidelines: bool = True):
         self.state = "pre"
@@ -827,11 +841,6 @@ class Game(GameNode):
     def board(self) -> chess.Board:
         return self.headers.board()
 
-    # TODO: Consider naming.
-    def _interactive_viewer(self) -> Any:
-        from chess._interactive import InteractiveViewer
-        return InteractiveViewer(self)  # type: ignore
-
     def ply(self) -> int:
         # Optimization: Parse FEN only for custom starting positions.
         return self.board().ply() if "FEN" in self.headers else 0
@@ -887,6 +896,14 @@ class Game(GameNode):
         visitor.end_game()
         return visitor.result()
 
+    def time_control(self) -> TimeControl:
+        """
+        Returns the time control of the game. If the game has no time control
+        information, the default time control ('UNKNOWN') is returned.
+        """
+        time_control_header = self.headers.get("TimeControl", "")
+        return parse_time_control(time_control_header)
+
     @classmethod
     def from_board(cls: Type[GameT], board: chess.Board) -> GameT:
         """Creates a game from the move stack of a :class:`~chess.Board()`."""
@@ -1000,10 +1017,10 @@ class Headers(MutableMapping[str, str]):
     def __len__(self) -> int:
         return len(self._tag_roster) + len(self._others)
 
-    def copy(self: HeadersT) -> HeadersT:
+    def copy(self) -> Self:
         return type(self)(self)
 
-    def __copy__(self: HeadersT) -> HeadersT:
+    def __copy__(self) -> Self:
         return self.copy()
 
     def __repr__(self) -> str:
@@ -1305,7 +1322,7 @@ class BoardBuilder(BaseVisitor[chess.Board]):
         return self.board
 
 
-class SkipVisitor(BaseVisitor[_TrueLiteral]):
+class SkipVisitor(BaseVisitor[Literal[True]]):
     """Skips a game."""
 
     def begin_game(self) -> SkipType:
@@ -1317,7 +1334,7 @@ class SkipVisitor(BaseVisitor[_TrueLiteral]):
     def begin_variation(self) -> SkipType:
         return SKIP
 
-    def result(self) -> _TrueLiteral:
+    def result(self) -> Literal[True]:
         return True
 
 
@@ -1791,3 +1808,61 @@ def skip_game(handle: TextIO) -> bool:
     Skips a game. Returns ``True`` if a game was found and skipped.
     """
     return bool(read_game(handle, Visitor=SkipVisitor))
+
+
+def parse_time_control(time_control: str) -> TimeControl:
+    tc = TimeControl()
+
+    if not time_control:
+        return tc
+
+    if time_control.startswith("?"):
+        return tc
+
+    if time_control.startswith("-"):
+        tc.type = TimeControlType.UNLIMITED
+        return tc
+
+    def _parse_part(part: str) -> TimeControlPart:
+        tcp = TimeControlPart()
+
+        moves_time, *bonus = part.split("+")
+
+        if bonus:
+            _bonus = bonus[0]
+            if _bonus.lower().endswith("d"):
+                tcp.delay = float(_bonus[:-1])
+            else:
+                tcp.increment = float(_bonus)
+
+        moves, *time = moves_time.split("/")
+        if time:
+            tcp.moves = int(moves)
+            tcp.time = int(time[0])
+        else:
+            tcp.moves = 0
+            tcp.time = int(moves)
+
+        return tcp
+
+    tc.parts = [_parse_part(part) for part in time_control.split(":")]
+
+    if len(tc.parts) > 1:
+        for part in tc.parts[:-1]:
+            if part.moves == 0:
+                raise ValueError("Only last part can be 'sudden death'.")
+
+    # Classification according to https://www.fide.com/FIDE/handbook/LawsOfChess.pdf
+    # (Bullet added)
+    base_time = tc.parts[0].time
+    increment = tc.parts[0].increment
+    if (base_time + 60 * increment) < 3 * 60:
+        tc.type = TimeControlType.BULLET
+    elif (base_time + 60 * increment) < 15 * 60:
+        tc.type = TimeControlType.BLITZ
+    elif (base_time + 60 * increment) < 60 * 60:
+        tc.type = TimeControlType.RAPID
+    else:
+        tc.type = TimeControlType.STANDARD
+
+    return tc
