@@ -1,19 +1,3 @@
-# This file is part of the python-chess library.
-# Copyright (C) 2012-2021 Niklas Fiekas <niklas.fiekas@backscattering.de>
-#
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program. If not, see <http://www.gnu.org/licenses/>.
-
 from __future__ import annotations
 
 import collections
@@ -28,7 +12,10 @@ import typing
 import chess
 
 from types import TracebackType
-from typing import Deque, Dict, Iterable, Iterator, List, Optional, Tuple, Type, TypeVar, Union
+from typing import Deque, Dict, Iterable, Iterator, List, Optional, Set, Tuple, Type, TypeVar, Union
+
+if typing.TYPE_CHECKING:
+    from typing_extensions import Self
 
 
 UINT64_BE = struct.Struct(">Q")
@@ -384,7 +371,7 @@ def is_tablename(name: str, *, one_king: bool = True, piece_count: Optional[int]
 def tablenames(*, one_king: bool = True, piece_count: int = 6) -> Iterator[str]:
     first = "K" if one_king else "P"
 
-    targets = []
+    targets: List[str] = []
 
     white_pieces = piece_count - 2
     black_pieces = 0
@@ -427,7 +414,7 @@ def _dependencies(target: str, *, one_king: bool = True) -> Iterator[str]:
 
 
 def dependencies(target: str, *, one_king: bool = True) -> Iterator[str]:
-    closed = set()
+    closed: Set[str] = set()
     if one_king:
         closed.add("KvK")
 
@@ -438,7 +425,7 @@ def dependencies(target: str, *, one_king: bool = True) -> Iterator[str]:
 
 
 def all_dependencies(targets: Iterable[str], *, one_king: bool = True) -> Iterator[str]:
-    closed = set()
+    closed: Set[str] = set()
     if one_king:
         closed.add("KvK")
 
@@ -548,8 +535,6 @@ class PawnFileDataDtz:
     norm: List[int]
 
 
-TableT = TypeVar("TableT", bound="Table")
-
 class Table:
     size: List[int]
 
@@ -612,7 +597,7 @@ class Table:
                 raise IOError(f"invalid file size: ensure {self.path!r} is a valid syzygy tablebase file")
 
             try:
-                # Python 3.8
+                # Unix
                 data.madvise(mmap.MADV_RANDOM)
             except AttributeError:
                 pass
@@ -700,10 +685,7 @@ class Table:
             i += norm[i]
 
     def calc_factors_piece(self, factor: List[int], order: int, norm: List[int]) -> int:
-        if not self.variant.connected_kings:
-            PIVFAC = [31332, 28056, 462]
-        else:
-            PIVFAC = [31332, 0, 518, 278]
+        PIVFAC = [31332, 0, 518, 278] if self.variant.connected_kings else [31332, 28056, 462]
 
         n = 64 - norm[0]
 
@@ -1044,7 +1026,7 @@ class Table:
                     self.data.close()
                     self.data = None
 
-    def __enter__(self: TableT) -> TableT:
+    def __enter__(self) -> Self:
         return self
 
     def __exit__(self, exc_type: Optional[Type[BaseException]], exc_value: Optional[BaseException], traceback: Optional[TracebackType]) -> None:
@@ -1273,14 +1255,14 @@ class DtzTable(Table):
             self.norm = [0 for _ in range(self.num)]
             self.tb_size = [0, 0, 0, 0]
             self.size = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
-            self.files = [PawnFileDataDtz() for f in range(4)]
+            self.files = [PawnFileDataDtz() for _ in range(4)]
 
             files = 4 if self.data[4] & 0x02 else 1
 
             p_data = 5
 
             if not self.has_pawns:
-                self.map_idx = [[0, 0, 0, 0]]
+                self.map_idx: List[List[int]] = [[0, 0, 0, 0]]
 
                 self.setup_pieces_piece_dtz(p_data, 0)
                 p_data += self.num + 1
@@ -1530,27 +1512,23 @@ class Tablebase:
 
         Returns the number of table files that were found.
         """
-        num = 0
         directory = os.path.abspath(directory)
+        return sum(self.add_file(os.path.join(directory, filename), load_wdl=load_wdl, load_dtz=load_dtz) for filename in os.listdir(directory))
 
-        for filename in os.listdir(directory):
-            path = os.path.join(directory, filename)
-            tablename, ext = os.path.splitext(filename)
-
-            if is_tablename(tablename, one_king=self.variant.one_king) and os.path.isfile(path):
-                if load_wdl:
-                    if ext == self.variant.tbw_suffix:
-                        num += self._open_table(self.wdl, WdlTable, path)
-                    elif "P" not in tablename and ext == self.variant.pawnless_tbw_suffix:
-                        num += self._open_table(self.wdl, WdlTable, path)
-
-                if load_dtz:
-                    if ext == self.variant.tbz_suffix:
-                        num += self._open_table(self.dtz, DtzTable, path)
-                    elif "P" not in tablename and ext == self.variant.pawnless_tbz_suffix:
-                        num += self._open_table(self.dtz, DtzTable, path)
-
-        return num
+    def add_file(self, path: str, *, load_wdl: bool = True, load_dtz: bool = True) -> int:
+        tablename, ext = os.path.splitext(os.path.basename(path))
+        if is_tablename(tablename, one_king=self.variant.one_king) and os.path.isfile(path):
+            if load_wdl:
+                if ext == self.variant.tbw_suffix:
+                    return self._open_table(self.wdl, WdlTable, path)
+                elif "P" not in tablename and ext == self.variant.pawnless_tbw_suffix:
+                    return self._open_table(self.wdl, WdlTable, path)
+            if load_dtz:
+                if ext == self.variant.tbz_suffix:
+                    return self._open_table(self.dtz, DtzTable, path)
+                elif "P" not in tablename and ext == self.variant.pawnless_tbz_suffix:
+                    return self._open_table(self.dtz, DtzTable, path)
+        return 0
 
     def probe_wdl_table(self, board: chess.Board) -> int:
         # Test for variant end.
@@ -1569,6 +1547,8 @@ class Tablebase:
         try:
             table = typing.cast(WdlTable, self.wdl[key])
         except KeyError:
+            if chess.popcount(board.occupied) > TBPIECES:
+                raise KeyError(f"syzygy tables support up to {TBPIECES} pieces, not {chess.popcount(board.occupied)}: {board.fen()}")
             raise MissingTableError(f"did not find wdl table {key}")
 
         self._bump_lru(table)
@@ -1581,7 +1561,12 @@ class Tablebase:
             raise KeyError(f"tablebase has been opened for {self.variant.uci_variant}, probed with: {board.uci_variant}")
         if board.castling_rights:
             raise KeyError(f"syzygy tables do not contain positions with castling rights: {board.fen()}")
-        if chess.popcount(board.occupied) > TBPIECES:
+
+        # Probing resolves captures, so sometimes we can obtain results for
+        # positions that have more pieces than the maximum number of supported
+        # pieces. We artificially limit this to one additional level, to
+        # make sure search remains somewhat bounded.
+        if chess.popcount(board.occupied) > TBPIECES + 1:
             raise KeyError(f"syzygy tables support up to {TBPIECES} pieces, not {chess.popcount(board.occupied)}: {board.fen()}")
 
         # Special case: Variant with compulsory captures.
